{
    "$graph": [
        {
            "class": "Workflow",
            "doc": "ChIP-seq 02 trimming - reads: SE",
            "requirements": [
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "doc": "Input adapters files",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "id": "#main/input_adapters_files"
                },
                {
                    "doc": "Input fastq files",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "id": "#main/input_read1_fastq_files"
                },
                {
                    "default": 1,
                    "doc": "Number of threads",
                    "type": "int",
                    "id": "#main/nthreads"
                },
                {
                    "default": "-phred33",
                    "type": "string",
                    "id": "#main/quality_score"
                },
                {
                    "default": "/usr/share/java/trimmomatic.jar",
                    "doc": "Trimmomatic Java jar file",
                    "type": "string",
                    "id": "#main/trimmomatic_jar_path"
                },
                {
                    "doc": "JVM arguments should be a quoted, space separated list",
                    "type": [
                        "null",
                        "string"
                    ],
                    "id": "#main/trimmomatic_java_opts"
                }
            ],
            "steps": [
                {
                    "run": "#count-fastq-reads.cwl",
                    "scatterMethod": "dotproduct",
                    "scatter": [
                        "#main/count_fastq_reads/input_fastq_file",
                        "#main/count_fastq_reads/input_basename"
                    ],
                    "in": [
                        {
                            "source": "#main/extract_basename/basename",
                            "id": "#main/count_fastq_reads/input_basename"
                        },
                        {
                            "source": "#main/trimmomatic/output_read1_trimmed_file",
                            "id": "#main/count_fastq_reads/input_fastq_file"
                        }
                    ],
                    "out": [
                        "#main/count_fastq_reads/output_read_count"
                    ],
                    "id": "#main/count_fastq_reads"
                },
                {
                    "run": "#basename.cwl",
                    "scatter": "#main/extract_basename/file_path",
                    "in": [
                        {
                            "valueFrom": "${return true}",
                            "id": "#main/extract_basename/do_not_escape_sep"
                        },
                        {
                            "source": "#main/trimmomatic/output_read1_trimmed_file",
                            "valueFrom": "$(self.basename)",
                            "id": "#main/extract_basename/file_path"
                        },
                        {
                            "valueFrom": "(\\.fastq.gz|\\.fastq)",
                            "id": "#main/extract_basename/sep"
                        }
                    ],
                    "out": [
                        "#main/extract_basename/basename"
                    ],
                    "id": "#main/extract_basename"
                },
                {
                    "run": "#trimmomatic.cwl",
                    "scatterMethod": "dotproduct",
                    "scatter": [
                        "#main/trimmomatic/input_read1_fastq_file",
                        "#main/trimmomatic/input_adapters_file"
                    ],
                    "in": [
                        {
                            "valueFrom": "SE",
                            "id": "#main/trimmomatic/end_mode"
                        },
                        {
                            "valueFrom": "2:30:15",
                            "id": "#main/trimmomatic/illuminaclip"
                        },
                        {
                            "source": "#main/input_adapters_files",
                            "id": "#main/trimmomatic/input_adapters_file"
                        },
                        {
                            "source": "#main/input_read1_fastq_files",
                            "id": "#main/trimmomatic/input_read1_fastq_file"
                        },
                        {
                            "source": "#main/trimmomatic_java_opts",
                            "id": "#main/trimmomatic/java_opts"
                        },
                        {
                            "valueFrom": "${return 3}",
                            "id": "#main/trimmomatic/leading"
                        },
                        {
                            "valueFrom": "${return 15}",
                            "id": "#main/trimmomatic/minlen"
                        },
                        {
                            "source": "#main/nthreads",
                            "id": "#main/trimmomatic/nthreads"
                        },
                        {
                            "valueFrom": "33",
                            "id": "#main/trimmomatic/phred"
                        },
                        {
                            "valueFrom": "4:20",
                            "id": "#main/trimmomatic/slidingwindow"
                        },
                        {
                            "valueFrom": "${return 3}",
                            "id": "#main/trimmomatic/trailing"
                        },
                        {
                            "source": "#main/trimmomatic_jar_path",
                            "id": "#main/trimmomatic/trimmomatic_jar_path"
                        }
                    ],
                    "out": [
                        "#main/trimmomatic/output_read1_trimmed_file"
                    ],
                    "id": "#main/trimmomatic"
                }
            ],
            "id": "#main",
            "outputs": [
                {
                    "doc": "Trimmed fastq files",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/trimmomatic/output_read1_trimmed_file",
                    "id": "#main/output_data_fastq_trimmed_files"
                },
                {
                    "doc": "Trimmed read counts of fastq files",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/count_fastq_reads/output_read_count",
                    "id": "#main/output_trimmed_fastq_read_count"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "doc": "Trimmomatic is a fast, multithreaded command line tool that can be used to trim and crop\nIllumina (FASTQ) data as well as to remove adapters. These adapters can pose a real problem\ndepending on the library preparation and downstream application.\nThere are two major modes of the program: Paired end mode and Single end mode. The\npaired end mode will maintain correspondence of read pairs and also use the additional\ninformation contained in paired reads to better find adapter or PCR primer fragments\nintroduced by the library preparation process.\nTrimmomatic works with FASTQ files (using phred + 33 or phred + 64 quality scores,\ndepending on the Illumina pipeline used).\n",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ShellCommandRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "dukegcb/trimmomatic",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 101,
                        "prefix": "AVGQUAL:",
                        "separate": false
                    },
                    "doc": "<quality>\nDrop the read if the average quality is below the specified level\n<quality>: Specifies the minimum average quality required to keep a read.\n",
                    "id": "#trimmomatic.cwl/avgqual"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 13,
                        "prefix": "CROP:",
                        "separate": false
                    },
                    "doc": "<length>\nRemoves bases regardless of quality from the end of the read, so that the read has maximally\nthe specified length after this step has been performed. Steps performed after CROP might of\ncourse further shorten the read.\n<length>: The number of bases to keep, from the start of the read.\n",
                    "id": "#trimmomatic.cwl/crop"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 3
                    },
                    "doc": "SE|PE\nSingle End (SE) or Paired End (PE) mode\n",
                    "id": "#trimmomatic.cwl/end_mode"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 13,
                        "prefix": "HEADCROP:",
                        "separate": false
                    },
                    "doc": "<length>\nRemoves the specified number of bases, regardless of quality, from the beginning of the read.\n<length>: The number of bases to keep, from the start of the read.\n",
                    "id": "#trimmomatic.cwl/headcrop"
                },
                {
                    "type": "string",
                    "doc": "<fastaWithAdaptersEtc>:<seed mismatches>:<palindrome clip threshold>:<simple clip threshold>:<minAdapterLength>:<keepBothReads>\nFind and remove Illumina adapters.\nREQUIRED:\n<fastaWithAdaptersEtc>: specifies the path to a fasta file containing all the adapters, PCR sequences etc.\nThe naming of the various sequences within this file determines how they are used. See below.\n<seedMismatches>: specifies the maximum mismatch count which will still allow a full match to be performed\n<palindromeClipThreshold>: specifies how accurate the match between the two 'adapter ligated' reads must be\nfor PE palindrome read alignment.\n<simpleClipThreshold>: specifies how accurate the match between any adapter etc. sequence must be against a read\nOPTIONAL:\n<minAdapterLength>: In addition to the alignment score, palindrome mode can verify\nthat a minimum length of adapter has been detected. If unspecified, this defaults to 8 bases,\nfor historical reasons. However, since palindrome mode has a very low false positive rate, this\ncan be safely reduced, even down to 1, to allow shorter adapter fragments to be removed.\n<keepBothReads>: After read-though has been detected by palindrome mode, and the\nadapter sequence removed, the reverse read contains the same sequence information as the\nforward read, albeit in reverse complement. For this reason, the default behaviour is to\nentirely drop the reverse read. By specifying \"true\" for this parameter, the reverse read will\nalso be retained, which may be useful e.g. if the downstream tools cannot handle a\ncombination of paired and unpaired reads.\n",
                    "id": "#trimmomatic.cwl/illuminaclip"
                },
                {
                    "type": "File",
                    "doc": "FASTA file containing adapters, PCR sequences, etc. It is used to search for and remove these sequences in the input FASTQ file(s)",
                    "id": "#trimmomatic.cwl/input_adapters_file"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "FASTQ file for input read (read R1 in Paired End mode)",
                    "id": "#trimmomatic.cwl/input_read1_fastq_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 6
                    },
                    "doc": "FASTQ file for read R2 in Paired End mode",
                    "id": "#trimmomatic.cwl/input_read2_fastq_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "shellQuote": false
                    },
                    "doc": "JVM arguments should be a quoted, space separated list (e.g. \"-Xms128m -Xmx512m\")",
                    "id": "#trimmomatic.cwl/java_opts"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 14,
                        "prefix": "LEADING:",
                        "separate": false
                    },
                    "doc": "<quality>\nRemove low quality bases from the beginning. As long as a base has a value below this\nthreshold the base is removed and the next base will be investigated.\n<quality>: Specifies the minimum quality required to keep a base.\n",
                    "id": "#trimmomatic.cwl/leading"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 4,
                        "prefix": "-trimlog"
                    },
                    "doc": "<ouptut log file name>\nSpecifying a trimlog file creates a log of all read trimmings, indicating the following details:\n  the read name\n  the surviving sequence length\n  the location of the first surviving base, aka. the amount trimmed from the start\n  the location of the last surviving base in the original read\n  the amount trimmed from the end\n<ouptut log file name>: filename for the generated output log file.\n",
                    "id": "#trimmomatic.cwl/log_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 15,
                        "prefix": "MAXINFO:",
                        "separate": false
                    },
                    "doc": "<targetLength>:<strictness>\nPerforms an adaptive quality trim, balancing the benefits of retaining longer reads against the\ncosts of retaining bases with errors.\n<targetLength>: This specifies the read length which is likely to allow the\nlocation of the read within the target sequence to be determined.\n<strictness>: This value, which should be set between 0 and 1, specifies the\nbalance between preserving as much read length as possible vs. removal of incorrect\nbases. A low value of this parameter (<0.2) favours longer reads, while a high value\n(>0.8) favours read correctness.\n",
                    "id": "#trimmomatic.cwl/maxinfo"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 100,
                        "prefix": "MINLEN:",
                        "separate": false
                    },
                    "doc": "<length>\nThis module removes reads that fall below the specified minimal length. If required, it should\nnormally be after all other processing steps. Reads removed by this step will be counted and\nincluded in the \"dropped reads\" count presented in the trimmomatic summary.\n<length>:  Specifies the minimum length of reads to be kept\n",
                    "id": "#trimmomatic.cwl/minlen"
                },
                {
                    "type": "int",
                    "default": 1,
                    "inputBinding": {
                        "position": 4,
                        "prefix": "-threads"
                    },
                    "doc": "Number of threads",
                    "id": "#trimmomatic.cwl/nthreads"
                },
                {
                    "type": "string",
                    "default": "64",
                    "inputBinding": {
                        "prefix": "-phred",
                        "separate": false,
                        "position": 4
                    },
                    "doc": "\"33\"|\"64\"\n-phred33 (\"33\") or -phred64 (\"64\") specifies the base quality encoding. Default: -phred64\n",
                    "id": "#trimmomatic.cwl/phred"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 15,
                        "prefix": "SLIDINGWINDOW:",
                        "separate": false
                    },
                    "doc": "<windowSize>:<requiredQuality>\nPerform a sliding window trimming, cutting once the average quality within the window falls\nbelow a threshold. By considering multiple bases, a single poor quality base will not cause the\nremoval of high quality data later in the read.\n<windowSize>: specifies the number of bases to average across\n<requiredQuality>: specifies the average quality required\n",
                    "id": "#trimmomatic.cwl/slidingwindow"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "TOPHRED33",
                        "separate": false
                    },
                    "doc": "This (re)encodes the quality part of the FASTQ file to base 33.",
                    "id": "#trimmomatic.cwl/tophred33"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "TOPHRED64",
                        "separate": false
                    },
                    "doc": "This (re)encodes the quality part of the FASTQ file to base 64.",
                    "id": "#trimmomatic.cwl/tophred64"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 14,
                        "prefix": "TRAILING:",
                        "separate": false
                    },
                    "doc": "<quality>\nRemove low quality bases from the end. As long as a base has a value below this threshold\nthe base is removed and the next base (which as trimmomatic is starting from the 3\" prime end\nwould be base preceding the just removed base) will be investigated. This approach can be\nused removing the special illumina \"low quality segment\" regions (which are marked with\nquality score of 2), but we recommend Sliding Window or MaxInfo instead\n<quality>: Specifies the minimum quality required to keep a base.\n",
                    "id": "#trimmomatic.cwl/trailing"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-jar"
                    },
                    "id": "#trimmomatic.cwl/trimmomatic_jar_path"
                }
            ],
            "outputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.log_filename)"
                    },
                    "doc": "Trimmomatic Log file.",
                    "id": "#trimmomatic.cwl/output_log_file"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.input_read1_fastq_file.path.replace(/^.*[\\\\\\/]/, '').replace(/(\\.fq.gz|\\.fastq.gz|\\.[^/.]+)$/, '.trimmed.fastq.gz'))"
                    },
                    "id": "#trimmomatic.cwl/output_read1_trimmed_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "${\n  if (inputs.end_mode == \"PE\")\n    return inputs.input_read1_fastq_file.path.replace(/^.*[\\\\\\/]/, '').replace(/(\\.fq.gz|\\.fastq.gz|\\.[^/.]+)$/, '.unpaired.trimmed.fastq.gz');\n  return null;\n}\n"
                    },
                    "id": "#trimmomatic.cwl/output_read1_trimmed_unpaired_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "${\n  if (inputs.end_mode == \"PE\" && inputs.input_read2_fastq_file)\n    return inputs.input_read2_fastq_file.path.replace(/^.*[\\\\\\/]/, '').replace(/(\\.fq.gz|\\.fastq.gz|\\.[^/.]+)$/, '.trimmed.fastq.gz');\n  return null;\n}\n"
                    },
                    "id": "#trimmomatic.cwl/output_read2_trimmed_paired_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "${\n  if (inputs.end_mode == \"PE\" && inputs.input_read2_fastq_file)\n    return inputs.input_read2_fastq_file.path.replace(/^.*[\\\\\\/]/, '').replace(/(\\.fq.gz|\\.fastq.gz|\\.[^/.]+)$/, '.unpaired.trimmed.fastq.gz');\n  return null;\n}\n"
                    },
                    "id": "#trimmomatic.cwl/output_read2_trimmed_unpaired_file"
                }
            ],
            "baseCommand": "java",
            "arguments": [
                {
                    "valueFrom": "$(\"-Djava.io.tmpdir=\"+runtime.tmpdir)",
                    "shellQuote": false,
                    "position": 1
                },
                {
                    "valueFrom": "$(inputs.input_read1_fastq_file.path.replace(/^.*[\\\\\\/]/, '').replace(/(\\.fq.gz|\\.fastq.gz|\\.[^/.]+)$/, '.trimmed.fastq.gz'))",
                    "position": 7
                },
                {
                    "valueFrom": "${\n  if (inputs.end_mode == \"PE\" && inputs.input_read2_fastq_file)\n    return inputs.input_read1_fastq_file.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, '') + '.trimmed.unpaired.fastq.gz';\n  return null;\n}\n",
                    "position": 8
                },
                {
                    "valueFrom": "${\n  if (inputs.end_mode == \"PE\" && inputs.input_read2_fastq_file)\n    return inputs.input_read2_fastq_file.path.replace(/^.*[\\\\\\/]/, '').replace(/(\\.fq.gz|\\.fastq.gz|\\.[^/.]+)$/, '.trimmed.fastq.gz');\n  return null;\n}\n",
                    "position": 9
                },
                {
                    "valueFrom": "${\n  if (inputs.end_mode == \"PE\" && inputs.input_read2_fastq_file)\n    return inputs.input_read2_fastq_file.path.replace(/^.*[\\\\\\/]/, '').replace(/(\\.fq.gz|\\.fastq.gz|\\.[^/.]+)$/, '.unpaired.trimmed.fastq.gz');\n  return null;\n}\n",
                    "position": 10
                },
                {
                    "valueFrom": "$(\"ILLUMINACLIP:\" + inputs.input_adapters_file.path + \":\"+ inputs.illuminaclip)",
                    "position": 11
                }
            ],
            "id": "#trimmomatic.cwl"
        },
        {
            "class": "ExpressionTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "boolean",
                    "default": false,
                    "id": "#basename.cwl/do_not_escape_sep"
                },
                {
                    "type": "string",
                    "id": "#basename.cwl/file_path"
                },
                {
                    "type": "int",
                    "default": 1,
                    "doc": "Number of file extensions to remove",
                    "id": "#basename.cwl/num_ext"
                },
                {
                    "type": "string",
                    "doc": "File extension delimiter",
                    "id": "#basename.cwl/sep"
                }
            ],
            "expression": "${\n  var sep_escaped = !inputs.do_not_escape_sep ? inputs.sep.replace(/[^\\w\\s]/g, \"\\\\$&\") : inputs.sep;\n  var out=inputs.file_path.replace(/^.*[\\\\\\/]/, '');\n  for (var e=0;e<inputs.num_ext; e++){\n    out=out.split(RegExp(\"\\\\b\" + sep_escaped + \"\\\\b\"))[0];\n  }\n  return {\"basename\": out};\n}\n",
            "outputs": [
                {
                    "type": "string",
                    "id": "#basename.cwl/basename"
                }
            ],
            "id": "#basename.cwl"
        },
        {
            "class": "CommandLineTool",
            "doc": "Counts reads in a fastq file",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "reddylab/workflow-utils:ggr",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "string",
                    "id": "#count-fastq-reads.cwl/input_basename"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1
                    },
                    "id": "#count-fastq-reads.cwl/input_fastq_file"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.input_basename + '.read_count.txt')"
                    },
                    "id": "#count-fastq-reads.cwl/output_read_count"
                }
            ],
            "baseCommand": "count-fastq-reads.sh",
            "stdout": "$(inputs.input_basename + '.read_count.txt')",
            "id": "#count-fastq-reads.cwl"
        }
    ],
    "cwlVersion": "v1.0"
}
